@@ -2,11 +2,13 @@ import copy
 import logging
 from multiprocessing.pool import ThreadPool as Pool
 
+from typing import Union
+
 import six
 import xmltodict
 
 from rtcclient import exception
-from rtcclient import urlparse, urlquote, OrderedDict
+from rtcclient import urlencode, urlparse, urlquote, OrderedDict
 from rtcclient.base import RTCBase
 from rtcclient.models import FiledAgainst, FoundIn, Comment, Action, State  # noqa: F401
 from rtcclient.models import IncludedInBuild, ChangeSet, Attachment  # noqa: F401
@@ -55,7 +57,10 @@ class RTCClient(RTCBase):
                  password,
                  proxies=None,
                  searchpath=None,
-                 ends_with_jazz=True):
+                 ends_with_jazz=True,
+                 verify: Union[bool, str] = False,
+                 old_rtc_authentication=False,
+                 **kwargs):
         """Initialization
 
         See params above
@@ -64,7 +69,9 @@ class RTCClient(RTCBase):
         self.username = username
         self.password = password
         self.proxies = proxies
-        RTCBase.__init__(self, url)
+        self.verify = verify
+        self.old_rtc_authentication = old_rtc_authentication
+        RTCBase.__init__(self, url, **kwargs)
 
         if not isinstance(ends_with_jazz, bool):
             raise exception.BadValue("ends_with_jazz is not boolean")
@@ -90,11 +97,29 @@ class RTCClient(RTCBase):
         _headers = {"Content-Type": self.CONTENT_XML}
         resp = self.get(self.url + "/authenticated/identity",
                         auth=(self.username, self.password),
-                        verify=False,
+                        verify=self.verify,
                         headers=_headers,
                         proxies=self.proxies,
                         allow_redirects=_allow_redirects)
 
+        if self.old_rtc_authentication:
+            # works with server that needs 0.6.0 version
+            _headers["Content-Type"] = self.CONTENT_URL_ENCODED
+            if resp.headers.get("set-cookie") is not None:
+                _headers["Cookie"] = resp.headers.get("set-cookie")
+
+            credentials = urlencode({
+                "j_username": self.username,
+                "j_password": self.password
+            })
+
+            resp = self.post(self.url + "/authenticated/j_security_check",
+                             data=credentials,
+                             verify=False,
+                             headers=_headers,
+                             proxies=self.proxies,
+                             allow_redirects=_allow_redirects)
+
         # authfailed
         authfailed = resp.headers.get("x-com-ibm-team-repository-web-auth-msg")
         if authfailed == "authfailed":
@@ -114,7 +139,7 @@ class RTCClient(RTCBase):
 
         resp = self.get(self.url + "/authenticated/identity",
                         auth=(self.username, self.password),
-                        verify=False,
+                        verify=self.verify,
                         headers=_headers,
                         proxies=self.proxies,
                         allow_redirects=_allow_redirects)
@@ -125,6 +150,42 @@ class RTCClient(RTCBase):
         else:
             _headers["Cookie"] = resp.headers.get("set-cookie")
 
+        # For Form Challenge
+        auth_msg = resp.headers.get("X-com-ibm-team-repository-web-auth-msg")
+        if auth_msg == "authrequired":
+            post_data = {
+                'j_username': self.username,
+                'j_password': self.password
+            }
+            temp_headers = {"Content-Type": self.CONTENT_URL_ENCODED}
+            resp = self.post(self.url + "/authenticated/j_security_check",
+                             data=post_data,
+                             verify=self.verify,
+                             headers=temp_headers,
+                             proxies=self.proxies,
+                             allow_redirects=True)
+
+            authfailed = resp.headers.get(
+                "x-com-ibm-team-repository-web-auth-msg")
+            if authfailed == "authfailed":
+                raise exception.RTCException("Authentication Failed: "
+                                             "Invalid username or password")
+            else:
+                if resp.status_code == 302:
+                    authfailedloc = resp.headers.get("Location")
+                    if authfailedloc is not None and authfailedloc in "/auth/authfailed":
+                        raise exception.RTCException(
+                            "Authentication Failed: "
+                            "Invalid username or password")
+                elif resp.status_code != 200:
+                    raise exception.RTCException("Authentication Failed: "
+                                                 "Invalid username or password")
+
+        if not _allow_redirects:
+            _headers["Cookie"] += "; " + resp.headers.get("set-cookie")
+        else:
+            _headers["Cookie"] = resp.headers.get("set-cookie")
+
         _headers["Accept"] = self.CONTENT_XML
         return _headers
 
@@ -920,13 +981,17 @@ class RTCClient(RTCBase):
 
         return self.templater.listFieldsFromWorkitem(copied_from, keep=keep)
 
-    def getWorkitem(self, workitem_id, returned_properties=None):
+    def getWorkitem(self,
+                    workitem_id,
+                    returned_properties=None,
+                    skip_full_attributes=True):
         """Get :class:`rtcclient.workitem.Workitem` object by its id/number
 
         :param workitem_id: the workitem id/number
             (integer or equivalent string)
         :param returned_properties: the returned properties that you want.
             Refer to :class:`rtcclient.client.RTCClient` for more explanations
+        :param skip_full_attributes: flag to retrieve all attributes.
         :return: the :class:`rtcclient.workitem.Workitem` object
         :rtype: rtcclient.workitem.Workitem
         """
@@ -950,7 +1015,7 @@ class RTCClient(RTCBase):
             else:
                 req_url = workitem_url
             resp = self.get(req_url,
-                            verify=False,
+                            verify=self.verify,
                             proxies=self.proxies,
                             headers=self.headers)
             raw_data = xmltodict.parse(resp.content)
@@ -959,7 +1024,8 @@ class RTCClient(RTCBase):
             return Workitem(workitem_url,
                             self,
                             workitem_id=workitem_id,
-                            raw_data=workitem_raw)
+                            raw_data=workitem_raw,
+                            skip_full_attributes=skip_full_attributes)
 
         except ValueError:
             excp_msg = "Please input a valid workitem id"
@@ -1175,7 +1241,7 @@ class RTCClient(RTCBase):
         headers['Content-Type'] = self.OSLC_CR_XML
 
         resp = self.post(url_post,
-                         verify=False,
+                         verify=self.verify,
                          headers=headers,
                          proxies=self.proxies,
                          data=workitem_raw)
@@ -1418,7 +1484,7 @@ class RTCClient(RTCBase):
                   if projectarea_id else None)
 
         resp = self.get(resource_url,
-                        verify=False,
+                        verify=self.verify,
                         proxies=self.proxies,
                         headers=self.headers)
         raw_data = xmltodict.parse(resp.content)
@@ -1454,18 +1520,19 @@ class RTCClient(RTCBase):
 
             # iterate all the entries
             with Pool() as p:
-                resources_list = list(
-                    filter(
-                        None,
-                        p.starmap(self._handle_resource_entry,
-                                  [(resource_name, entry, pa_url, archived,
-                                    filter_rule) for entry in entries])))
+                resources_list.extend(
+                    list(
+                        filter(
+                            None,
+                            p.starmap(self._handle_resource_entry,
+                                      [(resource_name, entry, pa_url, archived,
+                                        filter_rule) for entry in entries]))))
 
             # find the next page
             url_next = raw_data.get('oslc_cm:Collection').get('@oslc_cm:next')
             if url_next:
                 resp = self.get(url_next,
-                                verify=False,
+                                verify=self.verify,
                                 proxies=self.proxies,
                                 headers=self.headers)
                 raw_data = xmltodict.parse(resp.content)
