@@ -2,6 +2,8 @@ from rtcclient import RTCClient
 import requests
 import pytest
 import utils_test
+from unittest.mock import call
+
 from rtcclient.project_area import ProjectArea
 from rtcclient.models import Severity, Priority, FoundIn, FiledAgainst
 from rtcclient.models import TeamArea, Member, PlannedFor
@@ -42,6 +44,254 @@ def test_headers(mocker):
     assert client.headers == expected_headers
 
 
+def test_client_rest_calls_new_auth(mocker):
+    mocked_get = mocker.patch("requests.get")
+    mocked_post = mocker.patch("requests.post")
+
+    mock_rsp = mocker.MagicMock(spec=requests.Response)
+    mock_rsp.status_code = 200
+
+    mocked_get.return_value = mock_rsp
+    mocked_post.return_value = mock_rsp
+
+    mock_rsp.headers = {"set-cookie": "cookie-id"}
+    _ = RTCClient(url="http://test.url:9443/jazz",
+                  username="user",
+                  password="password")
+
+    # assert GET calls
+    assert mocked_get.call_count == 2
+    mocked_get.assert_has_calls([
+        call('http://test.url:9443/jazz/authenticated/identity',
+             verify=False,
+             headers={
+                 'Content-Type': 'text/xml',
+                 'Cookie': 'cookie-id',
+                 'Accept': 'text/xml'
+             },
+             proxies=None,
+             timeout=60,
+             auth=('user', 'password'),
+             allow_redirects=True),
+        call('http://test.url:9443/jazz/authenticated/identity',
+             verify=False,
+             headers={
+                 'Content-Type': 'text/xml',
+                 'Cookie': 'cookie-id',
+                 'Accept': 'text/xml'
+             },
+             proxies=None,
+             timeout=60,
+             auth=('user', 'password'),
+             allow_redirects=True)
+    ])
+
+    # assert POST calls
+    assert mocked_post.call_count == 0
+
+
+def test_client_rest_calls_old_auth(mocker):
+    mocked_get = mocker.patch("requests.get")
+    mocked_post = mocker.patch("requests.post")
+
+    mock_rsp = mocker.MagicMock(spec=requests.Response)
+    mock_rsp.status_code = 200
+
+    mocked_get.return_value = mock_rsp
+    mocked_post.return_value = mock_rsp
+
+    mock_rsp.headers = {"set-cookie": "cookie-id"}
+    _ = RTCClient(url="http://test.url:9443/jazz",
+                  username="user",
+                  password="password",
+                  old_rtc_authentication=True)
+
+    # assert GET calls
+    assert mocked_get.call_count == 2
+    mocked_get.assert_has_calls([
+        call('http://test.url:9443/jazz/authenticated/identity',
+             verify=False,
+             headers={
+                 'Content-Type': 'application/x-www-form-urlencoded',
+                 'Cookie': 'cookie-id',
+                 'Accept': 'text/xml'
+             },
+             proxies=None,
+             timeout=60,
+             auth=('user', 'password'),
+             allow_redirects=True),
+        call('http://test.url:9443/jazz/authenticated/identity',
+             verify=False,
+             headers={
+                 'Content-Type': 'application/x-www-form-urlencoded',
+                 'Cookie': 'cookie-id',
+                 'Accept': 'text/xml'
+             },
+             proxies=None,
+             timeout=60,
+             auth=('user', 'password'),
+             allow_redirects=True)
+    ])
+
+    # assert POST calls
+    assert mocked_post.call_count == 1
+    mocked_post.assert_has_calls([
+        call('http://test.url:9443/jazz/authenticated/j_security_check',
+             data='j_username=user&j_password=password',
+             json=None,
+             verify=False,
+             headers={
+                 'Content-Type': 'application/x-www-form-urlencoded',
+                 'Cookie': 'cookie-id',
+                 'Accept': 'text/xml'
+             },
+             proxies=None,
+             timeout=60,
+             allow_redirects=True)
+    ])
+
+
+def test_headers_auth_required_new_auth(mocker):
+    mocked_get = mocker.patch("requests.get")
+    mocked_post = mocker.patch("requests.post")
+
+    mock_rsp = mocker.MagicMock(spec=requests.Response)
+    mock_rsp.status_code = 200
+
+    # auth required
+    mock_rsp.headers = {
+        "set-cookie": "cookie-id",
+        "X-com-ibm-team-repository-web-auth-msg": "authrequired"
+    }
+
+    mocked_get.return_value = mock_rsp
+    mocked_post.return_value = mock_rsp
+
+    _ = RTCClient(url="http://test.url:9443/jazz",
+                  username="user",
+                  password="password")
+
+    # assert GET calls
+    assert mocked_get.call_count == 2
+    mocked_get.assert_has_calls([
+        call('http://test.url:9443/jazz/authenticated/identity',
+             verify=False,
+             headers={
+                 'Content-Type': 'text/xml',
+                 'Cookie': 'cookie-id',
+                 'Accept': 'text/xml'
+             },
+             proxies=None,
+             timeout=60,
+             auth=('user', 'password'),
+             allow_redirects=True),
+        call('http://test.url:9443/jazz/authenticated/identity',
+             verify=False,
+             headers={
+                 'Content-Type': 'text/xml',
+                 'Cookie': 'cookie-id',
+                 'Accept': 'text/xml'
+             },
+             proxies=None,
+             timeout=60,
+             auth=('user', 'password'),
+             allow_redirects=True)
+    ])
+
+    # assert POST calls
+    assert mocked_post.call_count == 1
+    mocked_post.assert_has_calls([
+        call('http://test.url:9443/jazz/authenticated/j_security_check',
+             data={
+                 'j_username': 'user',
+                 'j_password': 'password'
+             },
+             json=None,
+             verify=False,
+             headers={'Content-Type': 'application/x-www-form-urlencoded'},
+             proxies=None,
+             timeout=60,
+             allow_redirects=True)
+    ])
+
+
+def test_headers_auth_required_old_auth(mocker):
+    mocked_get = mocker.patch("requests.get")
+    mocked_post = mocker.patch("requests.post")
+
+    mock_rsp = mocker.MagicMock(spec=requests.Response)
+    mock_rsp.status_code = 200
+
+    # auth required
+    mock_rsp.headers = {
+        "set-cookie": "cookie-id",
+        "X-com-ibm-team-repository-web-auth-msg": "authrequired"
+    }
+
+    mocked_get.return_value = mock_rsp
+    mocked_post.return_value = mock_rsp
+
+    _ = RTCClient(url="http://test.url:9443/jazz",
+                  username="user",
+                  password="password",
+                  old_rtc_authentication=True)
+
+    # assert GET calls
+    assert mocked_get.call_count == 2
+    mocked_get.assert_has_calls([
+        call('http://test.url:9443/jazz/authenticated/identity',
+             verify=False,
+             headers={
+                 'Content-Type': 'application/x-www-form-urlencoded',
+                 'Cookie': 'cookie-id',
+                 'Accept': 'text/xml'
+             },
+             proxies=None,
+             timeout=60,
+             auth=('user', 'password'),
+             allow_redirects=True),
+        call('http://test.url:9443/jazz/authenticated/identity',
+             verify=False,
+             headers={
+                 'Content-Type': 'application/x-www-form-urlencoded',
+                 'Cookie': 'cookie-id',
+                 'Accept': 'text/xml'
+             },
+             proxies=None,
+             timeout=60,
+             auth=('user', 'password'),
+             allow_redirects=True)
+    ])
+
+    # assert POST calls
+    assert mocked_post.call_count == 2
+    mocked_post.assert_has_calls([
+        call('http://test.url:9443/jazz/authenticated/j_security_check',
+             data='j_username=user&j_password=password',
+             json=None,
+             verify=False,
+             headers={
+                 'Content-Type': 'application/x-www-form-urlencoded',
+                 'Cookie': 'cookie-id',
+                 'Accept': 'text/xml'
+             },
+             proxies=None,
+             timeout=60,
+             allow_redirects=True),
+        call('http://test.url:9443/jazz/authenticated/j_security_check',
+             data={
+                 'j_username': 'user',
+                 'j_password': 'password'
+             },
+             json=None,
+             verify=False,
+             headers={'Content-Type': 'application/x-www-form-urlencoded'},
+             proxies=None,
+             timeout=60,
+             allow_redirects=True)
+    ])
+
+
 class TestRTCClient:
 
     @pytest.fixture(autouse=True)
@@ -1376,6 +1626,73 @@ class TestRTCClient:
                               raw_data=utils_test.workitem1)
         assert workitem1 == workitem11
 
+    @pytest.mark.parametrize('item,expected', [
+        ('com.ibm.workitem.attribute.custom.completeperc', "0"),
+        ("com.ibm.team.workitem.linktype.parentworkitem.parent",
+         "input title here for 161"),
+        ("com.ibm.team.workitem.linktype.attachment.attachment",
+         "input title here for 161"),
+        ("com.ibm.team.workitem.linktype.resolvesworkitem.resolves",
+         "input title here for 161"),
+        ('com.ibm.team.workitem.linktype.relatedworkitem.related',
+         "input title here for 161"),
+        ('com.ibm.team.workitem.linktype.resolvesworkitem.resolves',
+         "input title here for 161"),
+        ("com.ibm.team.enterprise.promotion.linktype.promotionBuildResult.promotionBuildResult",
+         "input title here for 161"),
+        ("com.ibm.team.enterprise.deployment.linktype.deploymentDefinition.packageDefinition",
+         "input title here for 161"),
+        ("com.ibm.team.enterprise.deployment.linktype.deploymentBuildResult.packageBuildResult",
+         "input title here for 161"),
+        ("com.ibm.team.enterprise.promotion.linktype.resultWorkItem.promoted",
+         "input title here for 161"),
+        ("com.ibm.team.workitem.linktype.schedulePredecessor.predecessor",
+         "input title here for 161"),
+        ("com.ibm.team.build.linktype.includedPackages.com.ibm.team.build.common.link.includedInPackages",
+         "input title here for 161"),
+        ("com.ibm.team.enterprise.promotion.linktype.gapChangeSets.gapChangeSets",
+         "input title here for 161"),
+        ("com.ibm.team.workitem.linktype.resolvesworkitem.resolvedBy",
+         "input title here for 161"),
+        ("com.ibm.team.workitem.linktype.duplicateworkitem.duplicates",
+         "input title here for 161"),
+        ("com.ibm.team.workitem.linktype.parentworkitem.children",
+         "input title here for 161"),
+        ("com.ibm.team.enterprise.promotion.linktype.promotedBuildMaps.promotedBuildMaps",
+         "input title here for 161"),
+        ("com.ibm.team.workitem.linktype.schedulePredecessor.successor",
+         "input title here for 161"),
+        ("com.ibm.team.filesystem.workitems.change_set.com.ibm.team.scm.ChangeSet",
+         "input title here for 161"),
+        ("com.ibm.team.workitem.linktype.blocksworkitem.dependsOn",
+         "input title here for 161"),
+        ("com.ibm.team.enterprise.promotion.linktype.resultWorkItem.result",
+         "input title here for 161"),
+        ("com.ibm.team.workitem.linktype.relatedartifact.relatedArtifact",
+         "input title here for 161"),
+        ("com.ibm.workitem.attribute.custom.odc.deftype", None),
+        ("com.ibm.workitem.attribute.custom.odc.history", None),
+        ("com.ibm.workitem.attribute.custom.odc.qualifier", None),
+        ("com.ibm.workitem.attribute.custom.odc.trigger", None),
+        ("com.ibm.workitem.attribute.custome.isvalid", None)
+    ])
+    def test_get_workitem_full_attributes(self, mocker, myrtcclient, item,
+                                          expected):
+        mocked_get = mocker.patch("requests.get")
+        mock_resp = mocker.MagicMock(spec=requests.Response)
+        mock_resp.status_code = 200
+        mock_resp.content = utils_test.workitem1_raw
+        mocked_get.return_value = mock_resp
+
+        # Workitem1
+        workitem1 = Workitem("http://test.url:9443/jazz/oslc/workitems/161",
+                             myrtcclient,
+                             workitem_id=161,
+                             raw_data=utils_test.workitem1,
+                             skip_full_attributes=False)
+        assert workitem1.identifier == "161"
+        assert workitem1[item] == expected
+
     @pytest.fixture
     def mock_get_workitems(self, mocker):
         mocked_get = mocker.patch("requests.get")
